#  Function:     SKit4R
#  Version:      1.1
#  Date:         2012-10-26
#  Author:       Gabor Meszaros < gabor.meszaros@boku.ac.at > 
#  Maintainer:   Gabor Meszaros < gabor.meszaros@boku.ac.at > 
#  Depends:      R (> = 2.10) 
#  Description:  This is an over-arching function using coxWei4R,
#                prepare4R and sortSKit to run PREPARE, sorting (if needed) and 
#	             COX or WEIBULL in one click (given that the input parameters were 
#                correct. SKit4R is using the full range of options from prepare4R 
#                and coxWei4R (including the graphics), therefore it is necessary 
#                to have all functions in the working directory of R. 
#                Additionally SKit4R comes with a greatly simplified input for 
#                Survival Kit filenames, requiring only the name of the main data 
#                and pedigree file. In case of multiple computations it is possible 
#                to specify different result filenames, so these won't be overwritten by 
#                subsequent runs. 
#  License:      GPL   (version 3 or later)

SKit4R=function(
#
######################################
# initializing variables for prepare4R
######################################
#
  inputData="",       #name of the input data file - required!
  inputPedigree="",   #name of the input pedigree file
  inputSurvival="",     #name of the input file for FUTURE, to be used in the SURVIVAL later on
# typically you don't want to change these
  internalFiles=c("recInput.txt","recSorted.txt","codelist.txt","varlist.txt",
    "recFuture.txt","recPedigree.txt","readOldSol.txt",
    "storeSol.txt","logfile.txt"),
# in case of multiple runs, to distinguish between result filenames
  resultFiles=c("resultsMain.txt","resSurvival.txt","resResidual.txt"),
  pedigreeEffect="",
  effects="", 
  effectType="",
  classEffects=c("",""), 
  timeDepEffects=c(""),
  timeDepEffectsType=c(""),
  joinCodeEffects="",
  outputEffects="",
  time=c("life","",""),
  censName="cens",
  censValue=0,
  idRec="idnum",
  truncate="",
  discrete=FALSE,
  timeCovName="",
  timeCovType="",
  timeCovValues="",
  newEffectConvdt="",
  convdtEffects="",
  newEffectCombine="",
  combineEffects="", 
  formout="free_format",
#####################################
# initializing variables for coxWei4R
#####################################
#
  useR=TRUE,
  nrecmax=0,
  ndimax=0,  
  nefmax=0,
  nstramax=0,
  ntimmax=0,
  nstimax=0,
  nlevout=0,
  npgauss=0,
  niter_gauss=0,
  nvec_bf=0,
  no_logarithm=-1,
  no_hessian=FALSE,
  nze_hessian=0,
  with_mgd=FALSE,
  title="",
  ite_quasi=-1,
  strata="",        # connected with 'strataSort'
  origin="",
  rho_fixed=0,
  model=c("","",""),
  coefficient="",
  statistics=FALSE,
  only_statistics=FALSE,
  random="",
  integrate_out="",
  animal_solution="",
  std_error=TRUE,
  dense_hessian=FALSE,
  force_positive=FALSE,
  baseline=FALSE,
  kaplan=FALSE,
  moments=FALSE,
  survivalOptions="",
  constraintsOptions="",
  convergence=0,
  store_solutions=FALSE,
  read_old_solutions=FALSE,
  storage="IN_CORE",
  test="",
  correlation=c("","",""),
#
# input parameters for SKit4R function
#
  program="",
  UNIXlike=FALSE,
  verbose=FALSE,
  graphics=TRUE,
# connected with 'strata'
  strataSort=0,
#switch to use RESIDUAL in COX
  residual=FALSE,
# input parameters for changing the working directory of R (optional)
  wdNew="",
  wdSetBack=TRUE
  )
{

# Check for errors
if(is.null(inputData) | is.na(inputData) | inputData== "") stop("InputData has to be specified!")
if(strataSort==0 & strata!="") stop("If you use STRATA keyword for the Kit, 'strataSort' should be also set here!")
if(strataSort!=0 & strata=="") stop("If you use 'strataSort' here, the STRATA keyword should be set for the Kit!")

# Check for 'program'setting to run appropriate part of the function
if (program!="only_prepare" & program!="only_cox" &  program!="only_weibull" &
    program!="cox" & program!="weibull") stop("The program should be specified as '(only_)cox' or '(only_)weibull'")

###########
# Global filename structure PREPARE
###########
## Definition of empty vectors for filenames
 files=c("","","","") 
 futureFilenames=c("","")
 pedigreeFilenames=c("","")
## FILES
 files[1]=inputData
 files[2]=internalFiles[1]
 files[3]=internalFiles[3]
 files[4]=internalFiles[4]
## FUTURE - data preparation for SURVIVAL
 if (inputSurvival!="") {
   futureFilenames[1]=inputSurvival
   futureFilenames[2]=internalFiles[5]
 }
### Check for errors
 if (inputSurvival!="" & survivalOptions=="" | inputSurvival=="" & survivalOptions!="")
  stop("If you want to use SURVIVAL, both 'inputSurvival' and 'survivalOptions' should be specified")
## PEDIGREE
 if (inputPedigree!="") {
   pedigreeFilenames[1]=inputPedigree
   pedigreeFilenames[2]=internalFiles[6]
 }

#######################################################
# PREPARE
#######################################################
# reads in and runs PREPARE in all cases

  # if no "outputEffects" are specified prints out all available by default
  if (outputEffects=="") {
 #     outputEffects=as.vector(window(effects,deltat=2))
      outputEffects=as.vector(window(effects))
      # and attaches any "timeCov", "convdt" or "combine" variable created within Prepare 
      if (timeCovName!="") outputEffects=c(outputEffects,timeCovName)
      if (newEffectConvdt!="") outputEffects=c(outputEffects,newEffectConvdt)
      if (newEffectCombine!="") outputEffects=c(outputEffects,newEffectCombine)
  }  

#source("prepare4R.r")
prepare4R(nrecmax,nefmax,ndimax,files,futureFilenames,pedigreeFilenames,
  pedigreeEffect,effects,effectType,classEffects,timeDepEffects,timeDepEffectsType,joinCodeEffects,
  outputEffects,time,censName,censValue,idRec,truncate,discrete,
  timeCovName,timeCovType,timeCovValues,newEffectConvdt,convdtEffects,
  newEffectCombine,combineEffects,formout,correlation,
  UNIXlike,verbose,survivalOptions,inputPedigree,wdNew,wdSetBack,program)

#######################################################
# Sorting output file according to the requirements
#######################################################
#source("sortSKit.r")
 if (program=="weibull") {
   if (strataSort!=0 & strata!="") {
    #recoded data is sorted when strata is used in WEIBULL 
    sortSKit(inputFile=internalFiles[1],outputFile=internalFiles[2],
     order1=strataSort,wdNew=wdNew)
  }
 }
 if (program=="cox") {
  if (strataSort==0 & strata=="") {
   #no strata, normal case for COX
   sortSKit(inputFile=internalFiles[1],outputFile=internalFiles[2],
    mode="cox",wdNew=wdNew)
  }
  if (strataSort!=0 & strata!="") {
   #stratified COX model
   sortSKit(inputFile=internalFiles[1],outputFile=internalFiles[2],
    mode="cox",strata=strataSort,wdNew=wdNew)
  }
 }

#######################################################
# COX and WEIBULL
#######################################################
# set global input for COX and WEIBULL - the "program" variable 
#  decides how are these set and that which program will run

###########
# Global filename structure COX / WEIBULL
###########
## Definition of empty vectors for filenames
logfile=""
files=c("","","","","","") 
survivalFiles=c("","")
residualFiles=c("","")

## File to store the computation details output
logfile=internalFiles[9]

## FILES
if (program=="weibull" | program=="only_weibull") { 
 if (strataSort==0 & strata=="") files[1]=internalFiles[1] #weibull without strata
 if (strataSort!=0 & strata!="") files[1]=internalFiles[2] #weibull with strata
} 
if (program=="cox" | program=="only_cox") files[1]=internalFiles[2]     #cox with or without strata
files[2]=internalFiles[3]
files[3]=resultFiles[1]
if (inputPedigree!="") files[4]=internalFiles[6]
if (read_old_solutions) files[5]=internalFiles[7]
if (store_solutions) files[6]=internalFiles[8]

## SURVIVAL - if any options are specified the SURVIVAL is used
if (survivalOptions!="") {
 survivalFiles[1]=internalFiles[5]
 survivalFiles[2]=resultFiles[2]
}

## RESIDUAL
if (residual) {
 residualFiles[1]=internalFiles[1]
 residualFiles[2]=resultFiles[3]
}

# reads in and runs COX/WEIBULL
#source("coxWei4R.r")
coxWei4R(useR,nrecmax,ndimax,nefmax,nstramax,ntimmax,nstimax,nlevout,
  npgauss,niter_gauss,nvec_bf,no_logarithm,no_hessian,
  nze_hessian,with_mgd,logfile,files,title,ite_quasi,strata,origin,
  rho_fixed,model,coefficient,statistics,only_statistics,random,
  integrate_out,animal_solution,std_error,dense_hessian,force_positive,
  baseline,kaplan,moments,survivalFiles,survivalOptions,residualFiles,
  constraintsOptions,convergence,store_solutions,read_old_solutions,
  storage,test,correlation,
  program,UNIXlike,verbose,graphics,residual,wdNew,wdSetBack)
}
