#  Function:     coxWei4R
#  Version:      1.1
#  Date:         2012-10-26
#  Author:       Gabor Meszaros < gabor.meszaros@boku.ac.at > 
#  Maintainer:   Gabor Meszaros < gabor.meszaros@boku.ac.at > 
#  Depends:      R (> = 2.10) 
#  Description:  coxWei4R is to run either COX or WEIBULL part of the Survival Kit. 
#                It is part of the SKit4R function, but it can be used 
#                also separately (note the differences between SKit4R and coxWei4R 
#                in filename input). The "program" function argument decides which 
#                program will run, and sets the parameter and executable file names 
#                accordingly ('file' and 'exeFileName' in the code). It is also possible 
#                to plot certain functions using Survival Kit v6.07 or higher. 
#  License:      GPL   (version 3 or later)

coxWei4R= function (
# input parameters for the Survival Kit
  useR=TRUE,
  nrecmax=0,
  ndimax=0,  
  nefmax=0,
  nstramax=0,
  ntimmax=0,
  nstimax=0,
  nlevout=0,
  npgauss=0,
  niter_gauss=0,
  nvec_bf=0,
  no_logarithm=-1,
  no_hessian=FALSE,
  nze_hessian=0,
  with_mgd=FALSE,
  logfile="logfile.txt",
  files=c("","","","","",""),
  title="",
  ite_quasi=-1,
  strata="",
  origin="",
  rho_fixed=0,
  model=c("","",""),
  coefficient="",
  statistics=FALSE,
  only_statistics=FALSE,
  random="",
  integrate_out="",
  animal_solution="",
  std_error=TRUE,
  dense_hessian=FALSE,
  force_positive=FALSE,
  baseline=FALSE,
  kaplan=FALSE,
  moments=FALSE,
  survivalFiles="",
  survivalOptions="",
  residualFiles="",
  constraintsOptions="",
  convergence=0,
  store_solutions=FALSE,
  read_old_solutions=FALSE,
  storage="IN_CORE",
  test="",
  correlation=c("","",""),
#
# input parameters for this function
#
  program="",
  UNIXlike=FALSE,
  verbose=FALSE,
  graphics=TRUE,
#  survival=FALSE,    #switch to use SURVIVAL
  residual=FALSE,    #switch to use RESIDUAL in COX   
# input parameters for changing the working directory of R (optional)
#
  wdNew="",
  wdSetBack=TRUE
  )
{
# initial values
   wdCurr=getwd()
# changes working directory, if specified
   if (wdNew!="") setwd(wdNew)

#
# setting of the program executable and parameter file name
# WARNING: Requires standard input and exe file names to use this switch

  if (program=="cox" | program=="only_cox") {
       file="cox.txt"
       exeFileName="cox.exe"
  } else if (program=="weibull" | program=="only_weibull") {
       file="weibull.txt"
       exeFileName="weibull.exe"
  }
## in case of UNIX like systems (e.g. Linux) the exacutable needs a prefix to run
       if (UNIXlike) exeFileName=paste("./",exeFileName,sep="")

# functions to simplify the R code
## paste function when a vector of effects should be included
pasteCollapse=function(varName1) paste(varName1, collapse=" ")
## the usual case of a Survival Kit row
catPaste=function(varName2,varName3) cat(paste(varName2, varName3, ";"), "\n")
## when one needs to include just a single keyword
catPaste1=function(varName4) cat(paste(varName4, ";"), "\n")
## when one needs to put together three things in a row
catPaste3=function(varName5,varName6,varName7) cat(paste(varName5, varName6, varName7, ";"), "\n")
## when one needs to put together four things in a row
catPaste4=function(varName8,varName9,varName10,varName11) cat(paste(varName8, varName9, varName10, varName11, ";"), "\n")
## reporting error when using a COX keyword in WEIBULL
weiError=function(coxKeyword) if (program=="weibull" | program=="only_weibull") 
   stop("Keyword ",coxKeyword," is reserved for COX only!") 
## reporting error when using a WEIBULL keyword in COX
coxError=function(weiKeyword) if (program=="cox" | program=="only_cox") 
   stop("Keyword ",weiKeyword," is reserved for WEIBULL only!") 

# Check if the corect executable file is in working directory
  if (file.exists(exeFileName)==FALSE) stop("The file ",exeFileName," must be present in the working directory!")

  ## --- Ouput to file ---
  ## file - output file name, must not be NULL or empty
  ## verbose - logical, print output also in R
  sink(file=file, split=verbose)

# Checking if the keywords are used
# If yes, the appropriate lines are constructed 
#
  if (useR) catPaste1("USE_R")
  if (nrecmax > 0) catPaste("NRECMAX", format(nrecmax,scientific=F))
  if (ndimax > 0) catPaste("NDIMAX", ndimax)
  if (nefmax > 0) catPaste("NEFMAX", nefmax)
  if (nstramax > 0) catPaste("NSTRAMAX", nstramax)
  if (ntimmax > 0) catPaste("NTIMMAX", ntimmax)
  if (nstimax > 0) catPaste("NSTIMAX", nstimax)
  if (nlevout > 0) catPaste("NLEVOUT", nlevout)
  if (npgauss > 0) catPaste("NPGAUSS", npgauss)
  if (niter_gauss > 0) catPaste("NITER_GAUSS", niter_gauss) 
  if (nvec_bf > 0) catPaste("NVEC_BF", nvec_bf)
  if (no_logarithm == 0 | no_logarithm == 1) catPaste("NO_LOGARITHM", no_logarithm)
  if (no_hessian) catPaste1("NO_HESSIAN")
  if (nze_hessian > 0) catPaste("NZE_HESSIAN", nze_hessian)
  if (with_mgd) catPaste1("WITH_MGD")
  if (logfile != "") catPaste("LOGFILE", logfile)
  catPaste("FILES", pasteCollapse(files)) # FILES statement is always displayed
  if (title != "") catPaste("TITLE", title)
  if (ite_quasi >= 0) { catPaste("ITE_QUASI", ite_quasi)
                       coxError("ITE_QUASI") }
  if (strata != "") catPaste("STRATA", strata)
  if (origin != "") { catPaste("ORIGIN", origin)
                      coxError("ORIGIN") }
  if (rho_fixed > 0) { catPaste("RHO_FIXED", rho_fixed)
                       coxError("RHO_FIXED") }
  catPaste("MODEL", pasteCollapse(model)) # MODEL statement is always displayed
  if (coefficient  != "") catPaste("COEFFICIENT", pasteCollapse(coefficient))
  if (statistics) { catPaste1("STATISTICS")
                    coxError("STATISTICS") }
  if (only_statistics) { catPaste1("ONLY_STATISTICS")
                         coxError("ONLY_STATISTICS") }
  if (random != "") catPaste("RANDOM", random)
  if (integrate_out != "") { catPaste("INTEGRATE_OUT", integrate_out)
                             coxError("INTEGRATE_OUT") }
  if (animal_solution != "") { catPaste("ANIMAL_SOLUTION", animal_solution)
                               coxError("ANIMAL_SOLUTION") }
  if (std_error) catPaste1("STD_ERROR")
  if (dense_hessian) catPaste1("DENSE_HESSIAN")
  if (force_positive) catPaste1("FORCE_POSITIVE")
  if (baseline) { catPaste1("BASELINE")
                  weiError("BASELINE") }
  if (kaplan) { catPaste1("KAPLAN")
                weiError("KAPLAN") }
  if (any(survivalOptions != "")) 
       catPaste3("SURVIVAL", pasteCollapse(survivalFiles), pasteCollapse(survivalOptions))
  if (residual) { catPaste("RESIDUAL", pasteCollapse(residualFiles))
                                  weiError("RESIDUAL") }
  if (constraintsOptions != "") catPaste("CONSTRAINTS", constraintsOptions)
  if (convergence > 0) catPaste("CONVERGENCE_CRITERION", format(convergence,scientific=F))
  if (store_solutions) catPaste1("STORE_SOLUTIONS")
  if (read_old_solutions) catPaste1("READ_OLD_SOLUTIONS")
  if (storage != "") catPaste("STORAGE", storage)
  if (any(test != "")) catPaste("TEST", pasteCollapse(test))
  if (any(correlation!="")) catPaste("CORRELATION",pasteCollapse(correlation))

# End of parameter file creation
  sink()

# Check for potential errors
  if (no_hessian==TRUE & nze_hessian>0)
     stop("NO_HESSIAN and NZE_HESSIAN statements are contradictory. Choose just one.")
  if (statistics==TRUE & only_statistics==TRUE) 
     stop("Both STATISTICS and ONLY_STATISTICS requested. Choose just one.")
  if (survivalOptions!="" & residual==TRUE)
     stop("SURVIVAL and RESIDUAL can not be used in the same run.")   

# Calling the executable file for COX / WEIBULL
  system(exeFileName)

# Makes the plots in case 'useR' is specified (can be left out in case of 
#  older Survival Kit version - backward compatibility)
# In case the KAPLAN and/or BASELINE is specified and 
#  the user wants to make plots
# WARNING! In case of multiple plots the "Graphics history" should be enabled
#          in R, otherwise only the last plot will be displayed.
#
 if (useR){
# Color used for the main lines of the Survival Kit plots
# Change it, if you dont like it
  colUsed="blue"

 ######
 # Draws graphs related to RKaplan.txt 
 ######
  if (file.exists("RKaplan.txt") & kaplan==TRUE & graphics==TRUE) {
    RKaplan=read.table("RKaplan.txt")

# In case of no stratification
   if (strata==""){
# Plot of the Kaplan-Meier estiamte
    plot(x=RKaplan[,1],y=RKaplan[,2],type="s",xlab="Time",ylab="KM estimate",
         col=colUsed)
     lines(x=RKaplan[,1],y=RKaplan[,3],type="s",lty=4,col="red")
     lines(x=RKaplan[,1],y=RKaplan[,4],type="s",lty=4,col="red")
     title(main="Kaplan-Meier estiamte with 95% confidence intervals")

# Graphical test for Weibull - Plotting log(t) aganist log(-log S(t))
    xaxis=log(RKaplan[,1])
    yaxis=log(-log (RKaplan[,2]))
    plot(xaxis,yaxis,type="o",xlab="log ( time )",
     ylab="log ( -log KM estimate )",col=colUsed)
    title(main="Graphical test for Weibull model")
   } #endif (strata=="")
# In case of a stratified model
   if (strata!=""){
# initialize/reset variables
     i=2          # general counter variable
     strCount=0   # cunter for the number of strata
     strStart=1   # the row number on which the result for a strata start
     toPlot=FALSE # plotting switch

     while (i<=nrow(RKaplan)) {
      # find out strata change
      # if changes, turns on the switch
      if (RKaplan[i,1]<RKaplan[i-1,1]) {
        toPlot=TRUE
        # copies the relevant block into a temporary storage
        oneStrata=RKaplan[strStart:(i-1),]
        # sets a new starting point
        strStart=i
       }
       # when the program reached end of the file, turns the switch on
       if (i==nrow(RKaplan)) {
        toPlot=TRUE
        # copies the relevant block into a temporary storage
        oneStrata=RKaplan[strStart:nrow(RKaplan),]
       }
       # plots the results if the switch is on
       if(toPlot){
        # Counts stratas for the correct title
        strCount=strCount+1
     # Plot of the Kaplan-Meier estiamte
        titleText=paste('Kaplan-Meier estiamte with 95% confidence intervals for strata',strCount)
        plot(x=oneStrata[,1],y=oneStrata[,2],type="s",xlab="Time",ylab="KM estimate",
         col=colUsed)
         lines(x=oneStrata[,1],y=oneStrata[,3],type="s",lty=4,col="red")
         lines(x=oneStrata[,1],y=oneStrata[,4],type="s",lty=4,col="red")
         title(main=titleText)

     # Graphical test for Weibull - Plotting log(t) aganist log(-log S(t))
        titleText=paste('Graphical test for Weibull model for strata',strCount)
        xaxis=log(oneStrata[,1])
        yaxis=log(-log (oneStrata[,2]))
        plot(xaxis,yaxis,type="o",xlab="log ( time )",
         ylab="log ( -log KM estimate )",col=colUsed)
         title(main=titleText)
       # turns off the plotting switch
       toPlot=FALSE
       }
     i=i+1
    } #endWhile
   }  # endif (strata!="") 
  }   # endif RKaplan

 ######
 # Draws graphs related to RBaseline.txt 
 ######
  if (file.exists("RBaseline.txt") & baseline==TRUE & graphics==TRUE) {
    RBaseline=read.table("RBaseline.txt")

# In case of no stratification
   if (strata==""){
# Cumulative hazard function
    plot(x=RBaseline[,1],y=RBaseline[,2],type="s",xlab="Time",
     ylab="Cumulative Hazard",col=colUsed)
     title(main="Cumulative hazard function")

# Survival function
    plot(x=RBaseline[,1],y=RBaseline[,3],type="s",xlab="Time",
     ylab="Survival",col=colUsed)
     title(main="Survival function")
   } #endif (strata=="")
# In case of a stratified model
   if(strata!=""){
# initialize/reset variables
     i=2          # general counter variable
     strCount=0   # cunter for the number of strata
     strStart=1   # the row number on which the result for a strata start
     toPlot=FALSE # plotting switch

     while (i<=nrow(RBaseline)) {
      # find out strata change
      # if changes, turns on the switch
      if (RBaseline[i,1]<RBaseline[i-1,1]) {
        toPlot=TRUE
        # copies the relevant block into a temporary storage
        oneStrata=RBaseline[strStart:(i-1),]
        # sets a new starting point
        strStart=i
      }
      # when the program reached end of the file, turns the switch on
      if (i==nrow(RBaseline)) {
        toPlot=TRUE
        # copies the relevant block into a temporary storage
        oneStrata=RBaseline[strStart:nrow(RBaseline),]
      }
      # plots the results if the switch is on
      if(toPlot){
        # Counts stratas for the correct title
        strCount=strCount+1
     # Cumulativ hazard function
        titleText=paste('Cumulativ hazard function for strata',strCount)
        # plots the result             
        plot(x=oneStrata[,1],y=oneStrata[,2],type="s",xlab="Time",
         ylab="Cumulative Hazard",col=colUsed)
         title(main=titleText)
    # Survival function
        titleText=paste('Survival function for strata',strCount)
        plot(x=oneStrata[,1],y=oneStrata[,3],type="s",xlab="Time",
         ylab="Survival",col=colUsed)
         title(main=titleText)
         # turns off the plotting switch
        toPlot=FALSE
       }
     i=i+1
     }  # endWhile
   }    # endif(strata!="")
  }     # endBaseline

 ######
 # Draws the graphs related to RMoments.txt 
 ######
    if (file.exists("RMoments.txt") & moments==TRUE & graphics==TRUE) {
# reads in the file created by Survival Kit
     RMoments=scan("RMoments.txt",quiet=TRUE)
# surce in the function to R and print the graphs
#     source("GramChar.r")
     GramChar(RMoments[1],RMoments[2],RMoments[3],"Variable 1")
     if (RMoments[5]!=0) {
        GramChar(RMoments[4],RMoments[5],RMoments[6],"Variable 2")
     }
    } # endifMoments
	
 }      # endif (useR) 
 
# Warning message if 'useR' is not specified
 if (useR==FALSE) paste("Warning: The'useR' was set to FALSE - no plotting was done!")  
# setting back working directory to the original, if not specified otherwise
   if (wdSetBack) setwd(wdCurr)
}
