#  Function:     prepare4R
#  Version:      1.1
#  Date:         2012-10-26
#  Author:       Gabor Meszaros < gabor.meszaros@boku.ac.at > 
#  Maintainer:   Gabor Meszaros < gabor.meszaros@boku.ac.at > 
#  Depends:      R (> = 2.10) 
#  Description:  prepare4R is to run the PREPARE part of the Survival Kit. 
#                It is part of the SKit4R function, but it can be used also 
#                separately (note the differences between SKit4R and prepare4R 
#                in filename input). This function sorts according to the requirements 
#                of the COX program by default, assuming no stratification. If you plan 
#                to run a stratified weibull or cox model sort with sortSkit 
#                (or other software) accordingly, or use the SKit4R function.  
#  License:      GPL   (version 3 or later)

prepare4R=function(
# input parameters for the Survival Kit
  nrecmax=0,  
  nefmax=0,
  ndimax=0,
  files=c("","","",""),
  futureFilenames=c("",""),
  pedigreeFilenames=c("",""),
  pedigreeEffect="",
  effects=c("",""), 
  effectType=c("",""),
  classEffects=c("",""), 
  timeDepEffects=c(""),
  timeDepEffectsType=c(""),
  joinCodeEffects="",
  outputEffects="",
  time,
  censName,
  censValue,
  idRec,
  truncate="",
  discrete,
  timeCovName="",
  timeCovType="",
  timeCovValues="",
  newEffectConvdt="",
  convdtEffects="",
  newEffectCombine="",
  combineEffects="", 
  formout="free_format",
  correlation=c("","",""),
#
# input parameters for this function
#
  UNIXlike=FALSE,
  verbose=FALSE,
#  future=FALSE,    #switch to use FUTURE statement in PREPARE
  survivalOptions,   #switch to use FUTURE statement in PREPARE, link with SURVIVAL
  inputPedigree,    #switch to use PEDIGREE statement in PREPARE
#  pedigree=FALSE,  #switch to use PEDIGREE statement in PREPARE
#
# input parameters for changing the working directory of R (optional)
#
  wdNew="",
  wdSetBack=TRUE,
  program
  )
{
# initial values
   wdCurr=getwd()
# changes working directory, if specified
   if (wdNew!="") setwd(wdNew)  

# functions to simplify the R code
## paste function when a vector of effects should be included
pasteCollapse=function(varName1) paste(varName1, collapse=" ")
## the usual case of a Survival Kit row
catPaste=function(varName2,varName3) cat(paste(varName2, varName3, ";"), "\n")
## when one needs to include just a single keyword
catPaste1=function(varName4) cat(paste(varName4, ";"), "\n")
## when one needs to put together three things in a row
catPaste3=function(varName5,varName6,varName7) cat(paste(varName5, varName6, varName7, ";"), "\n")
## when one needs to put together four things in a row
catPaste4=function(varName8,varName9,varName10,varName11) cat(paste(varName8, varName9, varName10, varName11, ";"), "\n")

# default filenames
     exeFileName="prepare.exe"
     file="prepare.txt"
## in case of UNIX like systems (e.g. Linux) the exacutable needs a prefix to run
       if (UNIXlike) exeFileName=paste("./",exeFileName,sep="")

# Checking if the obligatory keywords for SK (FILES, INPUT, TIME, CENSCODE and OUTPUT are present
  if(is.null(file) | is.na(file) | file == "") stop("Filename has to be specified")
  if(any(is.null(files) | is.na(files) | files == "")) stop("File names in FILES statement have to be specified")
  if(any(is.null(effects) | is.na(effects) | effects== "")) stop("Effects have to be specified")
#  if(is.null(time[1]) | is.na(time[1]) | time[1] == "") stop("The TIME variable has to be specified")
  if(is.null(censName) | is.na(censName) | censName== "") stop("The name of the censoring variable has to be specified")
  if(is.null(censValue) | is.na(censValue) | censValue== "") stop("The value defining censored variables has to be specified")
  if(any(is.null(outputEffects) | is.na(outputEffects) | outputEffects== "")) stop("Effects have to be specified")
# Check if the corect executable file is in working directory
  if (file.exists(exeFileName)==FALSE) stop("The file ",exeFileName," must be present in the working directory!")

# Processing the "effects" making the I4, R4,... more understandable
effects=as.character(effects)
# case 1: all class effects, no action needed
     if (effectType[1]=="") { 
       ## creating SK signs
       effectTypeSK=as.character(rep("I4",length(effects)))
       ## joining vectors
       effectsSK=paste(effects,effectTypeSK,sep=" ")
     }
# case 2: at least one of the effects is real-R4, date6-D6, or date8-D8 
     if (effectType[1]!="") { 
       ## potential error messages
       if (any(effectType!= "class" & effectType!= "continuous" & effectType!= "date6" & effectType!= "date8") ) stop("The effect types should be specified as class, continuous, date6 or date8!")
       if (length(effects)!=length(effectType)) stop("Effect type specifications should be equal to the number of effects!")
       ## replacing according to SK specifications
       effectType=gsub("class","I4",effectType)
       effectType=gsub("continuous","R4",effectType)
       effectType=gsub("date6","D6",effectType)
       effectType=gsub("date8","D8",effectType)
       ## joining vectors
       effectsSK=paste(effects,effectType,sep=" ")
     }

###
# The same for time dependent effects "effects"
timeDepEffects=as.character(timeDepEffects)
# case 1: all class effects, no action needed
     if (timeDepEffectsType[1]=="") { 
       ## creating SK signs
       timeDepEffectsTypeSK=as.character(rep("I4",length(timeDepEffects)))
       ## joining vectors
       timeDepEffectsSK=paste(timeDepEffects,timeDepEffectsTypeSK,sep=" ")
     }
# case 2: at least one of the timeDepEffects is real-R4, date6-D6, or date8-D8 
     if (timeDepEffectsType[1]!="") { 
       ## potential error messages
       if (any(timeDepEffectsType!= "class" & timeDepEffectsType!= "continuous" & timeDepEffectsType!= "date6" & timeDepEffectsType!= "date8") ) stop("The time dependent effect types should be specified as class, continuous, date6 or date8!")
       if (length(timeDepEffects)!=length(timeDepEffectsType)) stop("The time dependent effect type specifications should be equal to the number of time dependent effects!")
       ## replacing according to SK specifications
       timeDepEffectsType=gsub("class","I4",timeDepEffectsType)
       timeDepEffectsType=gsub("continuous","R4",timeDepEffectsType)
       timeDepEffectsType=gsub("date6","D6",timeDepEffectsType)
       timeDepEffectsType=gsub("date8","D8",timeDepEffectsType)
       ## joining vectors
       timeDepEffectsSK=paste(timeDepEffects,timeDepEffectsType,sep=" ")
     }
###

  ## --- Ouput to file ---
  ## file - output file name, must not be NULL or empty
  ## verbose - logical, print output also in R
  sink(file=file, split=verbose)

# Checking if the keywords are used
# If yes, the appropriate lines are constructed 
#
  if (nrecmax > 0) catPaste("NRECMAX", format(nrecmax,scientific=F))
  if (nefmax > 0) catPaste("NEFMAX", nefmax)
  if (ndimax > 0) catPaste("NDIMAX", ndimax)
  if (any(files != "" | !is.na(files))) catPaste("FILES", pasteCollapse(files))
  if (any(effects!= "" | !is.na(effects))) catPaste("INPUT", pasteCollapse(effectsSK))
  if (time[1]!="") catPaste("TIME",pasteCollapse(time))
  if (censName!="") catPaste3("CENSCODE",censName,censValue)
  if (idRec!="") catPaste("IDREC",idRec)
  if (truncate!="") catPaste("TRUNCATE",truncate)
  if (discrete) catPaste1("DISCRETE_SCALE")
  if (timeCovName!="") catPaste4("TIMECOV",timeCovName,timeCovType,pasteCollapse(timeCovValues))
  if (newEffectConvdt!="") cat(paste("CONVDT",newEffectConvdt,"=",convdtEffects[1],
                              "-",convdtEffects[2],";"), "\n")
    # in the current version only max. 1 CONVDT equation possible
  if (any(classEffects!= "")) catPaste("CLASS",pasteCollapse(classEffects))
  if (any(timeDepEffects!= "")) catPaste("TIMEDEP",pasteCollapse(timeDepEffectsSK))
  if (newEffectCombine!="") cat(paste("COMBINE",newEffectCombine,"=",combineEffects[1],
                              "+",combineEffects[2],";"), "\n")
    # in the current version only max. 1 COMBINE equation possible
  if (any(joinCodeEffects!= "")) catPaste("JOINCODE",pasteCollapse(joinCodeEffects))
  if (any(outputEffects!= "" | !is.na(outputEffects)))  catPaste("OUTPUT",pasteCollapse(outputEffects)) 
 # if (future) catPaste("FUTURE",pasteCollapse(futureFilenames))
  if (survivalOptions!="") catPaste("FUTURE",pasteCollapse(futureFilenames))
  if (inputPedigree!="") catPaste3("PEDIGREE",pasteCollapse(pedigreeFilenames),pasteCollapse(pedigreeEffect))
  if (formout!="") catPaste("FORMOUT",formout)
  if (any(correlation!="")) catPaste3("CORRELATION",correlation[1],correlation[2])

# End of parameter file creation
  sink()

# Calling the executable file for PREPARE, when specified
  if (program!="only_cox" & program!="only_weibull") system(exeFileName)

# Fixing a Linux specific issue with output file format
  if (UNIXlike) {
## counting the number of time dependent effects in the output 
    timDepInOutput=0
    for (i in 1: length(timeDepEffects)) {
      if (any(timeDepEffects[i]==outputEffects)) timDepInOutput=timDepInOutput+1
    }
## counting the number of time dependent effects in the combine statement and then output
    timDepInCombine=0
    for (i in 1: length(timeDepEffects)) {
      if (any(timeDepEffects[i]==combineEffects)) {
        if (any(newEffectCombine==outputEffects)) timDepInCombine=timDepInCombine+1
      } 
    }
## sum up and output
    colNumber=length(outputEffects)+timDepInOutput+timDepInCombine
    recIn=as.data.frame(matrix(scan(files[2]),ncol=colNumber,byrow=TRUE))
    write.table(recIn,files[2],quote=FALSE,row.names=FALSE,col.names=FALSE)
  }

# setting back working directory to the original, if not specified otherwise
   if (wdSetBack) setwd(wdCurr)
}
