# This is to link program simul.f into R
# Original Fortran program by: Vincent Ducrocq, INRA, France
# Modifications for R by: Gabor Meszaros, BOKU, Austria

# DESCRIPTION:
#  This program will generate a data set "datsim" with N 
#  (possibly censored) records from a Weibull distribution.
#  Up to 3 fixed effects and one random (normal) "sire" effect 
#  are allowed. A simple relationship matrix (through sires of sires)
#  can be included. True parameter values are stored in file "true".

# USAGE (example):
#simul(n=62,rho=1.5,tmed=500,nfix=2,nlevfix=c(5,10,0),fixmin=c(0.7,0.9,0),fixmax=c(1.5,2.2,0),
#nsire=15,variance=0.04,nss=5,wdNew="d:/temporer/simultest")

simul = function(
                seed=1234,        #seed for random number generator
                                  #has a default value
                n=0,              #number of records
                rho=0,            #Weibull (rho) parameter
                                  # rho = 1: exponential; 
                                  # rho > 1: increasing hazard;
                                  # rho < 1 : decreasing hazard
                tmed=0,           #median survival time in days
                tcens=0,          #0 if no censoring
                nfix=0,           #number of fixed effects (up to 3)
                nlevfix=c(0,0,0), #number of levels per fixed effect
                fixmin=c(0,0,0),  #lower risk ratio for the fixed effect
                fixmax=c(0,0,0),  #upper risk ratio for the fixed effect
                nsire=0,          #random effect - number of sires (max 500)
                                  #0 if no random effect
                                  #if > 0 then random effect included
                variance=0,       #true variance for the random effect
                nss=0,            #related sires - number of sires sires
                                  #0 if no random effect
                                  #if > 0 then relationship matrix included
                wdNew=" ",        #option to specify the location of the dll file
                                  # by default: the working directory of R
                wdSetBack=TRUE    #if FALSE the directory from 'wdNew' is kept
                                  # by default: set back to the original wd of R
                )
{
# changes working directory, if specified
   wdCurr=getwd()
   if (nchar(wdNew)>1){
         setwd(wdNew)
                      }
   wdUsed=getwd() 
   location=paste(wdUsed,"/simul.dll",sep="")
# loads the compiled library from the specified path
#dyn.load("d:/work/R_work_files/simul/simul.dll")
dyn.load(location)

# general error messages
if (seed<=0) {
  stop("Error: The seed number has to be greater then 0!
The current value is: ", seed)
             }
if (n<=0) {
  stop("Error: Number of records to generate has to be greater then 0!
The current value is: ", n)
          }
if (rho<=0) {
  stop("Error: The Weibull parameter (rho) has to be greater then 0!
The current value is: ", rho)
            }
if (tmed<=0) {
  stop("Error: The median survival time has to be greater then 0!
The current value is: ", tmed)
             }
if (tcens<0) {
  stop("Error: The censoring time has to be greater then 0 or equal to 0
 in case of no censoring! The current value is: ", tcens)
             }
## error messages for fixed effects
if (nfix<1 | nfix>3) {
  stop("Error: The number of fixed effects should be between 1 and 3!
The current value is: ", nfix)
                     }
for (i in 1:nfix){
  if (fixmin[i]>=fixmax[i]) {
    stop ("The upper risk ratio for the fixed effects should be greater than the lower risk ratio!
The current values (min, max) are: ",fixmin[i]," and ",fixmax[i] )
                            }
  if (fixmin[i]<0 | fixmax[i]<0) {
    stop ("All risk ratios should be greater than 0!")
                                 }
  if (fixmin[i]==0 | fixmax[i]==0) {
    stop (nfix," min and max risk ratios expected to be greater than 0
One of these is problematic: ",fixmin[i]," or ",fixmax[i] )
                                   }
  if (nlevfix[i]<=0) {
    stop ("Error: Problem with the number of levels for effect ",i,"
 This should be a positive integer. The current value is: ",nlevfix[i] )
                                   }
                 }

## error messages for random effects 
if (nsire<0 | nsire>500) {
  stop("Error: Number of sires should be maximally 500 
or 0 in case of no random effect! The currect value is: ",nsire)
               }
if (nsire!=0 & variance==0) {
  stop("Error: No variance specified for the random sire effect!")
               }
if (variance<0 | variance>1) {
  stop("Error: The variance should be a real number between 0 and 1!")
               }
if (nss<0 | nss>100) {
 stop("Error: Number of sires sires should be maximally 100
or 0 in case of no relationships between sires! The currect value is: ",nss)
               }

# runs the subroutine and unloads the dll file
# 'result' is a dummy variable to supress output
result=.Fortran("simul",as.integer(seed),as.integer(n),as.double(rho),
   as.double(tmed),as.integer(tcens),as.integer(nfix),
   as.integer(nlevfix),as.double(fixmin),as.double(fixmax),
   as.integer(nsire),as.double(variance),as.integer(nss))

#dyn.unload("d:/work/R_work_files/simul/simul.dll")
dyn.unload(location)

# back to initial values, if not specified otherwise
   if (wdSetBack==TRUE){
      setwd(wdCurr)
                       }
}

